/*-------------------------------------------------------------------------
 *
 * hash_funcs.c
 *		Set of functions for HMAC.
 *
 * Copyright (c) 1996-2020, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
 *		  hash_funcs/hash_funcs.c
 *
 *-------------------------------------------------------------------------
 */

#include "postgres.h"
#include "fmgr.h"

#include "common/cryptohash.h"
#include "common/md5.h"
#include "common/sha2.h"

PG_MODULE_MAGIC;

/*
 * This is the hash_funcs function.
 */
PG_FUNCTION_INFO_V1(hash_md5);
PG_FUNCTION_INFO_V1(hash_sha256);

Datum
hash_md5(PG_FUNCTION_ARGS)
{
	bytea      *arg = PG_GETARG_BYTEA_PP(0);
	const uint8 *data;
	size_t      len;
	pg_cryptohash_ctx *ctx;
	unsigned char buf[MD5_DIGEST_LENGTH];
	bytea      *result;

	len = VARSIZE_ANY_EXHDR(arg);
	data = (unsigned char *) VARDATA_ANY(arg);

	ctx = pg_cryptohash_create(PG_MD5);
	if (pg_cryptohash_init(ctx) < 0)
		elog(ERROR, "could not initialize %s context", "MD5");
	if (pg_cryptohash_update(ctx, data, len) < 0)
		elog(ERROR, "could not update %s context", "MD5");
	if (pg_cryptohash_final(ctx, buf) < 0)
		elog(ERROR, "could not finalize %s context", "MD5");
	pg_cryptohash_free(ctx);

	result = palloc(sizeof(buf) + VARHDRSZ);
	SET_VARSIZE(result, sizeof(buf) + VARHDRSZ);
	memcpy(VARDATA(result), buf, sizeof(buf));

	PG_RETURN_BYTEA_P(result);
}

Datum
hash_sha256(PG_FUNCTION_ARGS)
{
	bytea      *arg = PG_GETARG_BYTEA_PP(0);
	const uint8 *data;
	size_t      len;
	pg_cryptohash_ctx *ctx;
	unsigned char buf[PG_SHA256_DIGEST_LENGTH];
	bytea      *result;

	len = VARSIZE_ANY_EXHDR(arg);
	data = (unsigned char *) VARDATA_ANY(arg);

	ctx = pg_cryptohash_create(PG_SHA256);
	if (pg_cryptohash_init(ctx) < 0)
		elog(ERROR, "could not initialize %s context", "SHA256");
	if (pg_cryptohash_update(ctx, data, len) < 0)
		elog(ERROR, "could not update %s context", "SHA256");
	if (pg_cryptohash_final(ctx, buf) < 0)
		elog(ERROR, "could not finalize %s context", "SHA256");
	pg_cryptohash_free(ctx);

	result = palloc(sizeof(buf) + VARHDRSZ);
	SET_VARSIZE(result, sizeof(buf) + VARHDRSZ);
	memcpy(VARDATA(result), buf, sizeof(buf));

	PG_RETURN_BYTEA_P(result);
}
